/**
	$Id $

	Copyright (C) 2014 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.extensions.login;

/**
	Information about a the login/password fields on the login form.
	
	@author Nicklas
	@since 3.3
*/
public class FieldInfo 
{

	/**
		This is the default login field.
	*/
	public static final FieldInfo DEFAULT_LOGIN = new FieldInfo("Login", false);
	
	/**
		This is the default password field.
	*/
	public static final FieldInfo DEFAULT_PASSWORD = new FieldInfo("Password", true);
	
	private String prompt;
	private String tooltip;
	private String placeHolder;
	private String clazz;
	private String style;
	private boolean hiddenCharacters;
	
	public FieldInfo()
	{}
	
	public FieldInfo(String prompt, boolean hiddenCharacters)
	{
		this.prompt = prompt;
		this.hiddenCharacters = hiddenCharacters;
	}
	
	/**
		Get the prompt that is associated with the field.
	*/
	public String getPrompt()
	{
		return prompt;
	}
	public void setPrompt(String prompt)
	{
		this.prompt = prompt;
	}
	
	/**
		Get a tooltip text that is displayed when the user holds the mouse over
		the input field.
	*/
	public String getTooltip()
	{
		return tooltip;
	}
	public void setTooltip(String tooltip)
	{
		this.tooltip = tooltip;
	}
	
	/**
		Get a placeholder text that is displayed in the field when it is empty.
	*/
	public String getPlaceHolder()
	{
		return placeHolder;
	}
	public void setPlaceHolder(String placeHolder)
	{
		this.placeHolder = placeHolder;
	}
	
	/**
		Get a class attribute that can used to style the input field.
	*/
	public String getClazz()
	{
		return clazz;
	}
	public void setClazz(String clazz)
	{
		this.clazz = clazz;
	}
	
	/**
		Get a style attribute that can used to style the input field.
	*/
	public String getStyle()
	{
		return style;
	}
	public void setStyle(String style)
	{
		this.style = style;
	}
	
	/**
		If set, the input fields has hidden characters, otherwise they are visible.
	*/
	public boolean hasHiddenCharacters()
	{
		return hiddenCharacters;
	}
	
	public void setHasHiddenCharacters(boolean hiddenCharacters)
	{
		this.hiddenCharacters = hiddenCharacters;
	}

}
