/*
	$Id: Stylesheets.java 6881 2015-04-21 07:59:42Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.taglib.extensions;

import java.io.IOException;
import java.io.NotSerializableException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Collection;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.tagext.TagSupport;

import net.sf.basedb.clients.web.extensions.JspContext;
import net.sf.basedb.clients.web.taglib.Head;

/**
	Tag for creating &lt;link&gt; tags to stylesheets 
	from a {@link JspContext}.

	<p>
	<b>Syntax:</b>
	<pre>
   &lt;ext:stylesheets 
      context=...
  /&gt;
</pre>

	<table border=1 cellspacing=0 cellpadding=2>
	<caption>Tag attributes</caption>
	<tr>
		<td><b>Attribute</b></td>
		<td><b>Default value</b></td>
		<td><b>Required</b></td>
		<td><b>Description</b></td>
	</tr>
	<tr>
		<td>context</td>
		<td>-</td>
		<td>yes</td>
		<td>
			A {@link JspContext} to pick stylesheets from.
		</td>
	</tr>
	</table>

	@author Nicklas
	@version 2.7
*/
public class Stylesheets 
	extends TagSupport
{

	private static final long serialVersionUID = -3549659176356969880L;

	// The JSP context
	private JspContext context = null;

	public void setContext(JspContext context)
	{
		this.context = context;
	}

	@Override
	public int doStartTag() 
		throws JspException
	{
		if (context == null) return SKIP_BODY;

		Collection<String> stylesheets = context.getStylesheets();
		if (stylesheets == null) return SKIP_BODY;
		
		StringBuilder sb =new StringBuilder();
		for (String stylesheet : stylesheets)
		{
			char pp = stylesheet.indexOf('?') == -1 ? '?' : '&';
			sb.append("<link rel=\"stylesheet\" type=\"text/css\" href=\"");
			sb.append(stylesheet).append(pp).append(Head.getVersionParameter(stylesheet));
			sb.append("\">\n");
		}
		
		try
		{
			pageContext.getOut().print(sb.toString());
    	}
		catch (Exception ex)
		{
			throw new JspTagException(ex.getMessage());
		}
		return SKIP_BODY;
	}
	
	// This class is not serializable
	private void readObject(ObjectInputStream ois)
		throws ClassNotFoundException, IOException 
	{
		throw new NotSerializableException();
	}
	private void writeObject(ObjectOutputStream ois)
		throws IOException 
	{
		throw new NotSerializableException();
	}

}


