/*
	$Id: BioMaterialData.java 6758 2015-02-23 08:43:11Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import java.util.Set;

/**
	This class is the root class for biomaterials.

	@author Nicklas
	@version 2.0
	@see net.sf.basedb.core.BioMaterial
	@see <a href="../../../../../../html/developer/api/data_api.biomaterials.html">Developer documentation: Biomaterials LIMS</a>
	@base.modified $Date: 2015-02-23 09:43:11 +0100 (må, 23 feb 2015) $
	@hibernate.class table="`BioMaterials`" lazy="true" discriminator-value="-1"
	@hibernate.discriminator column="`discriminator`" type="int"
*/
public abstract class BioMaterialData
	extends AnnotatedData
	implements SubtypableData
{

	public BioMaterialData()
	{}
	
	/**
		It may happen that lazy-loaded proxies are put in the same collection as 
		already loaded "real" objects in which case the BasicData.equals() method
		doesns't work. If both objects are BioMaterialData objects we can safely
		compare the id since they all share the same database table and id:s are 
		always unique even for items of different types.
		@since 3.3
	*/
	@Override
	public boolean equals(Object other) 
	{
		if (other instanceof BioMaterialData)
		{
			int thisId = this.getId();
			int otherId = ((BioMaterialData)other).getId();
			return thisId == 0 ? this == other : otherId == thisId;
		}
		else
		{
			return super.equals(other);
		}
	}

	/*
		From the SubtypableData interface
		----------------------------------
	*/
	private ItemSubtypeData subtype;
	@Override
	public ItemSubtypeData getItemSubtype()
	{
	   return subtype;
	}
	@Override
	public void setItemSubtype(ItemSubtypeData subtype)
	{
	   this.subtype = subtype;
	}
	// ------------------------------

	
	/**
		The maximum length of the external id that can be stored in the database.
		@see #setExternalId(String)
	*/
	public static final int MAX_EXTERNAL_ID_LENGTH = 255;
	private String externalId;
	/**
		Get the external id for the biomaterial
		@hibernate.property column="`external_id`" type="string" length="255" not-null="false"
	*/
	public String getExternalId()
	{
		return externalId;
	}
	public void setExternalId(String externalId)
	{
		this.externalId = externalId;
	}

	private Set<MeasuredBioMaterialData> children;
	/**
		This is the inverse end. NOTE! This collection will not 
		contained pooled children or hybridizations.
		@see MeasuredBioMaterialData#getParent()
		@since 2.9
		@hibernate.set inverse="true" lazy="true"
		@hibernate.collection-key column="`parent_id`"
		@hibernate.collection-one-to-many class="net.sf.basedb.core.data.MeasuredBioMaterialData"
	*/
	Set<MeasuredBioMaterialData> getChildren()
	{
		return children;
	}
	void setChildren(Set<MeasuredBioMaterialData> children)
	{
		this.children = children;
	}

	private Set<BioMaterialEventSourceData> childCreationEvents;
	/**
		This is the inverse end.
		@see BioMaterialEventSourceData#getBioMaterial()
		@hibernate.set lazy="true" inverse="true"
		@hibernate.collection-key column="`biomaterial_id`"
		@hibernate.collection-one-to-many class="net.sf.basedb.core.data.BioMaterialEventSourceData"
	*/
	Set<BioMaterialEventSourceData> getChildCreationEvents()
	{
		return childCreationEvents;
	}
	void setChildCreationEvents(Set<BioMaterialEventSourceData> childCreationEvents)
	{
		this.childCreationEvents = childCreationEvents;
	}

}
