/**
	$Id: InterruptHandler.java 5405 2010-09-10 11:09:05Z nicklas $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.signal;

/**
	An interface for handling thread interrupts. A handler is responsible for
	taking some kind of action when a thread has been interrupted via 
	{@link Thread#interrupt()}. Handlers are registered with {@link 
	ThreadSignalHandler#setInterruptHandler(InterruptHandler)} and are
	invoked from {@link ThreadSignalHandler#checkInterrupted()} when the
	currently running thread has been interrupted.
	
	<p>
	Note to BASE developers! Long running operations inside the BASE 
	core or utilities should call {@link ThreadSignalHandler#checkInterrupted()}
	at regular intervals to make sure that operations can be interrupted.

	@author Nicklas
	@since 2.16
	@base.modified $Date: 2010-09-10 13:09:05 +0200 (fr, 10 sep 2010) $
*/
public interface InterruptHandler
{

	/**
		This method is called when the worker thread has been interrupted by
		some other thread.
	*/
	public void handleInterrupt();
	
}
