/**
	$Id: ExtensionsFilter.java 4515 2008-09-11 20:08:14Z jari $

	Copyright (C) Authors contributing to this file.

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.extensions;

import java.util.List;

/**
	Allows extensions and extension points to be enabled/disabled
	and sorted externally. An object of this class is used by the
	{@link Registry#useExtensions(ClientContext, ExtensionsFilter, String...)}
	method.

	@author nicklas
	@version 2.7
	@base.modified $Date:2008-03-20 12:15:25 +0100 (Thu, 20 Mar 2008) $
*/
public interface ExtensionsFilter 
{

	/**
		Check if an extension point is enabled or disabled.
		@param extensionPoint The extension point to check
		@return TRUE if the extension point is enabled, FALSE if 
			it is disabled
	*/
	public boolean isEnabled(ExtensionPoint<?> extensionPoint);
	
	/**
		Check if an extension is enabled or disabled.
		@param extension The extension  to check
		@return TRUE if the extension is enabled, FALSE if 
			it is disabled
	*/
	public boolean isEnabled(Extension<?> extension);
	
	/**
		Sort a list of invoked extensions according to some property.
		@param extensions The list of extensions to sort
	*/
	public void sort(List<? extends InvokationContext<?>> extensions);
	
}
