/**
	$Id$

	Copyright (C) 2015 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.listable;

import java.util.Set;

import net.sf.basedb.core.Extract;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Restrictions;

/**
	Source item transformer implementation that transform physical bioassay items 
	to extracts. This implementation will only transform to the physical bioassay:s
	immediate parent extract. Grandparents are not included. To do this, combine this
	transformed with a {@link ExtractToParentExtractTransformer}.
	
	@author Nicklas
	@since 3.5
*/
public class PhysicalBioAssayToExtractTransformer 
	extends AbstractSourceItemTransformer
{

	/**
		Create a new physical bioassay to extract transformer.
	*/
	public PhysicalBioAssayToExtractTransformer() 
	{
		super(Item.PHYSICALBIOASSAY, Item.EXTRACT);
	}
	
	@Override
	public Set<Integer> transform(TransformContext context, Set<Integer> source) 
	{
		ItemQuery<Extract> query = Extract.getQuery();
		query.setIncludes(context.getInclude());
		query.join(Hql.innerJoin("childCreationEvents", "ce"));
		query.join(Hql.innerJoin("ce", "event", "evt"));
		query.join(Hql.innerJoin("evt", "physicalBioAssay", "pba"));
		query.restrict(
			Restrictions.in(
				Hql.alias("pba"), 
				Expressions.parameter("bioAssays")
			)
		);
		return safeIdList(context.getDbControl(), query, "bioAssays", source);
	}


}
