/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2012 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/

var Color = function()
{
	var colorField;
	
	var color = {};

	color.initPage = function()
	{
		// Get the current color from the parent window
		colorField = window.opener.document.getElementById(Data.get('page-data', 'textarea'));
		
		var colorValue = colorField.value;
		// Split into r,g,b values
		var rgb = colorValue.match('([0-9A-Fa-f]{2})([0-9A-Fa-f]{2})([0-9A-Fa-f]{2})');
		
		// Put r,g,b values in input fields
		var frm = document.forms['color'];
		frm.red.value = rgb[1];
		frm.green.value = rgb[2];
		frm.blue.value = rgb[3];
		
		// Initialize color boxes
		var titleToFocus = '#'+rgb[0];
		for (var x = 0; x < 18; x++)
		{
			for (var y = 0; y < 12; y++)
			{
				var colorBox = Doc.element('x'+x+'y'+y);
				Buttons.addClickHandler(colorBox, color.colorOnClick);
				Events.addEventHandler(colorBox, 'keypress', color.handleArrowKeys);
				
				if (colorBox.title == titleToFocus)
				{
					colorBox.focus();
				}
			}
		}
		
		// Dialog buttons
		Buttons.addClickHandler('close', App.closeWindow);
		Buttons.addClickHandler('btnSave', color.save);
	}
	
	/**
		Save the selected color back to the parent
		form and close the window.
	*/
	color.save = function()
	{
		var frm = document.forms['color'];
		if (!color.validateColorValue(frm.red.value))
		{
			Forms.showNotification(frm.red, 'Invalid red color value');
			return;
		}
		else if (!color.validateColorValue(frm.green.value))
		{
			Forms.showNotification(frm.green, 'Invalid green color value');
			return;
		}
		else if (!color.validateColorValue(frm.blue.value))
		{
			Forms.showNotification(frm.blue, 'Invalid blue color value');
			return;
		}
		// Save the value to the date field
		colorField.value = frm.red.value + frm.green.value + frm.blue.value;
		// Fire an 'onchange' event
		Events.sendChangeEvent(colorField);

		window.close();
	}
	
	/**
		Event handler for selecting a color among the
		color boxes.
	*/
	color.colorOnClick = function(event)
	{
		var target = event.currentTarget;
		var frm = document.forms['color'];
		frm.red.value = Data.get(target, 'red');
		frm.green.value = Data.get(target, 'green');
		frm.blue.value = Data.get(target, 'blue');
	}
	
	
	/**
		Set focus to 'next' color when arrow keys are pressed.
	*/
	color.handleArrowKeys = function(event)
	{
		var keyCode = event.keyCode;
		if (keyCode >= 37 && keyCode <= 40)
		{
			var y = Data.int(event.currentTarget, 'y');
			var x = Data.int(event.currentTarget, 'x');
			// 37 - left, 38 - up, 39 - right, 40 - down
			if (keyCode == 37) x--;
			if (keyCode == 38) y--;
			if (keyCode == 39) x++;
			if (keyCode == 40) y++;
			var next = Doc.element('x'+x+'y'+y);
			if (next && next.tabIndex != -1)
			{
				next.focus();
			}
		}
	}
	
	/**
		A color value must be two characters using
		0-9 or A-F.
	*/
	color.validateColorValue = 	function(value)
	{
		return value.match('[0-9A-Fa-f]{2}');
	}
	
	return color;
}();

Doc.onLoad(Color.initPage);


